# pylint: disable=protected-access,redefined-outer-name,no-value-for-parameter
"""Test to verify that Beward library works."""

import pytest
import requests_mock

from beward import Beward, BewardDoorbell

from . import function_url
from .const import MOCK_HOST, MOCK_PASS, MOCK_USER


def test_factory():
    """Test that factory method works."""
    with requests_mock.Mocker() as mock:
        mock.register_uri(
            "get", function_url("systeminfo"), text="DeviceModel=NONEXISTENT"
        )

        with pytest.raises(ValueError):  # noqa: PT011
            Beward.factory(MOCK_HOST, MOCK_USER, MOCK_PASS)

        # ruff: noqa: ERA001, TD002
        # TODO: BewardCamera; pylint: disable=fixme
        # 123
        # mock.register_uri("get", function_url('systeminfo'),
        #                   text='DeviceModel=????')
        #
        # res = Beward.factory(MOCK_HOST, MOCK_USER, MOCK_PASS)
        # assert isinstance(res, BewardCamera) is True

        mock.register_uri("get", function_url("systeminfo"), text="DeviceModel=DS06M")

        beward = Beward.factory(MOCK_HOST, MOCK_USER, MOCK_PASS)
        assert isinstance(beward, BewardDoorbell) is True



# pylint: disable=protected-access,redefined-outer-name
"""Test to verify that Beward library works."""

from datetime import datetime

import requests_mock

from beward import BewardDoorbell
from beward.const import ALARM_SENSOR

from . import function_url, load_binary
from .const import MOCK_HOST, MOCK_PASS, MOCK_USER, local_tz


def test__handle_alarm():
    """Test that handle alarms."""
    image = load_binary("image.jpg")

    with requests_mock.Mocker() as mock:
        beward = BewardDoorbell(MOCK_HOST, MOCK_USER, MOCK_PASS)

        # Check initial state
        assert beward.last_motion_timestamp is None
        assert beward.last_motion_image is None

        ts1 = datetime.now(local_tz)
        mock.register_uri(
            "get",
            function_url("images"),
            content=image,
            headers={"Content-Type": "image/jpeg"},
        )
        beward._handle_alarm(ts1, ALARM_SENSOR, state=True)
        assert beward.last_ding_timestamp == ts1
        assert beward.last_ding_image == image



#  Copyright (c) 2019-2022, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
"""Beward camera controller."""

import logging
from datetime import datetime
from typing import Any

from requests import ConnectTimeout

from beward.const import ALARM_MOTION

from .core import BewardGeneric

_LOGGER = logging.getLogger(__name__)


class BewardCamera(BewardGeneric):
    """Beward camera controller class."""

    # pylint: disable=too-many-arguments
    def __init__(
        self,
        host: str,
        username: str,
        password: str,
        rtsp_port: int | None = None,
        stream: int = 0,
        **kwargs: Any,
    ) -> None:
        """Initialize Beward camera controller."""
        super().__init__(host, username, password, **kwargs)

        self.last_motion_timestamp = None
        self.last_motion_image = None

        self.rtsp_port = rtsp_port
        self.stream = stream

        self._live_image_url = None
        self._rtsp_live_video_url = None

    def obtain_uris(self) -> None:
        """Set the URIs for the camera."""
        self._live_image_url = self.get_url(
            "images",
            extra_params={"channel": 0},
            # Add authentication data
            username=self.username,
            password=self.password,
        )

        if not self.rtsp_port:
            try:
                info = self.get_info("rtsp")
                self.rtsp_port = info.get("RtspPort", 554)
            except ConnectTimeout:
                self.rtsp_port = 554

        self._rtsp_live_video_url = (
            f"rtsp://{self.username}:{self.password}@"
            f"{self.host}:{self.rtsp_port}/av0_{self.stream}"
        )

    @property
    def live_image_url(self) -> str:
        """Return URL to get live photo from camera."""
        if not self._live_image_url:
            self.obtain_uris()
        return self._live_image_url

    @property
    def rtsp_live_video_url(self) -> str:
        """Return URL to get live video from camera via RTSP protocol."""
        if not self._live_image_url:
            self.obtain_uris()
        return self._rtsp_live_video_url

    @property
    # pylint: disable=unsubscriptable-object
    def live_image(self) -> bytes | None:
        """Return bytes of camera image."""
        res = self.query("images", extra_params={"channel": 0})

        if res.headers.get("Content-Type") not in ("image/jpeg", "image/png"):
            return None

        return res.content

    def _handle_alarm(self, timestamp: datetime, alarm: str, state: bool) -> None:  # noqa: FBT001
        """Handle alarms from Beward device."""
        super()._handle_alarm(timestamp, alarm, state)

        if alarm == ALARM_MOTION and state:
            self.last_motion_timestamp = timestamp
            self.last_motion_image = self.live_image



#  Copyright (c) 2019-2024, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
"""Constants."""

from importlib import metadata

# Base library constants
VERSION = "1.1.13"

mdata = metadata.metadata(__package__)

URLS = dict(x.split(", ") for x in mdata.get_all("Project-URL"))

STARTUP_MESSAGE = f"""
-------------------------------------------------------------------
{mdata["Summary"]}
Version: {mdata["Version"]}
If you have ANY issues with this you need to open an issue here:
{URLS["Issues"]}
-------------------------------------------------------------------
"""

TIMEOUT = 3

# Error strings
MSG_GENERIC_FAIL = "Sorry.. Something went wrong..."

# Alarms
ALARM_ONLINE = "DeviceOnline"
ALARM_MOTION = "MotionDetection"
ALARM_SENSOR = "SensorAlarm"
ALARM_SENSOR_OUT = "SensorOutAlarm"

# Device types
BEWARD_CAMERA = "camera"
BEWARD_DOORBELL = "doorbell"

# Beward models
BEWARD_MODELS = {
    BEWARD_CAMERA: "B102S",
    BEWARD_DOORBELL: "ALP-600 DC11EP DK103 DK103M DKS20210 DS03M DS05M(P) DS06 DS06M "
    "DS06A(P) DS07P-LP DSN06PS DSN23215PS S06A S06M",
}



# pylint: disable=protected-access,redefined-outer-name,no-value-for-parameter
"""Test to verify that Beward library works."""

import logging
from datetime import datetime
from time import sleep

import pytest
import requests
import requests_mock

from beward import BewardGeneric
from beward.const import (
    ALARM_MOTION,
    ALARM_ONLINE,
    ALARM_SENSOR,
    BEWARD_CAMERA,
    BEWARD_DOORBELL,
)

from . import function_url, load_fixture
from .const import MOCK_HOST, MOCK_PASS, MOCK_USER, local_tz


@pytest.mark.parametrize("host", ["265.265.265.265", "-1.-1.-1.-1"])
def test___init__failing(host) -> None:
    """Test class initialization failing."""
    with pytest.raises(ValueError):  # noqa: PT011
        BewardGeneric(host, MOCK_USER, MOCK_PASS)


@pytest.mark.parametrize(
    ("host", "port", "host_exp", "port_exp"),
    [
        (MOCK_HOST, None, MOCK_HOST, 80),
        (MOCK_HOST + ":123", None, MOCK_HOST, 123),
        (MOCK_HOST, 456, MOCK_HOST, 456),
    ],
)
def test___init__(host: str, port, host_exp: str, port_exp: int) -> None:
    """Test class initialization."""
    bwd = BewardGeneric(host, MOCK_USER, MOCK_PASS, port=port)
    assert bwd.host == host_exp
    assert bwd.port == port_exp


@pytest.mark.parametrize(
    ("device", "dev_type"),
    [
        ("B102S", BEWARD_CAMERA),
        ("DS03M", BEWARD_DOORBELL),
        ("DS06M", BEWARD_DOORBELL),
        (None, None),
        ("NONEXISTENT", None),
    ],
)
def test_get_device_type(device: str, dev_type: str) -> None:
    """Test that correct device type detection."""
    assert BewardGeneric.get_device_type(device) == dev_type


def test_get_url(beward) -> None:
    """Test that correct URLs composing."""
    expect = f"http://{MOCK_HOST}:80/cgi-bin/systeminfo_cgi"
    res = beward.get_url("systeminfo")
    #
    assert res == expect

    expect = f"http://{MOCK_HOST}:80/cgi-bin/systeminfo_cgi?arg=123"
    res = beward.get_url("systeminfo", extra_params={"arg": "123"})
    #
    assert res == expect

    username = "user"
    expect = f"http://{username}@{MOCK_HOST}:80/cgi-bin/systeminfo_cgi"
    res = beward.get_url("systeminfo", username=username)
    #
    assert res == expect

    username = "user"
    password = "pass"  # noqa: S105
    expect = f"http://{username}:{password}@{MOCK_HOST}:80/cgi-bin/systeminfo_cgi"
    res = beward.get_url("systeminfo", username=username, password=password)
    #
    assert res == expect

    beward = BewardGeneric(MOCK_HOST, MOCK_USER, MOCK_PASS, port=123)

    expect = f"http://{MOCK_HOST}:123/cgi-bin/systeminfo_cgi"
    res = beward.get_url("systeminfo")
    #
    assert res == expect

    username = "user"
    expect = f"http://{username}@{MOCK_HOST}:123/cgi-bin/systeminfo_cgi"
    res = beward.get_url("systeminfo", username=username)
    #
    assert res == expect


def test_add_url_param(beward) -> None:
    """Test that correct URLs parameters adding."""
    base_url = f"http://{MOCK_HOST}/cgi-bin/systeminfo_cgi"

    expect = base_url + "?arg=123"
    res = beward.add_url_params(base_url, {"arg": "123"})
    #
    assert res == expect


def test_query(caplog) -> None:
    """Test that send requests to device."""
    caplog.set_level(logging.DEBUG)
    function = "systeminfo"

    with requests_mock.Mocker() as mock:
        beward = BewardGeneric(MOCK_HOST, MOCK_USER, MOCK_PASS)

        caplog.clear()
        expect = load_fixture("systeminfo.txt")
        mock.register_uri("get", function_url(function), text=expect)

        res = beward.query(function)

        assert res.text == expect
        assert next((s for s in caplog.messages if MOCK_PASS in s), None) is None

        caplog.clear()
        expect = "test data"
        mock.register_uri("get", function_url(function) + "?extra=123", text=expect)

        res = beward.query(function, extra_params={"extra": 123})

        assert res.text == expect
        assert next((s for s in caplog.messages if MOCK_PASS in s), None) is None


def test_add_alarms_handler(beward) -> None:
    """Test that add alarm handlers."""
    # Check initial state
    assert beward._alarm_handlers == set()

    def logger1() -> None:
        pass

    def logger2() -> None:
        pass

    beward.add_alarms_handler(logger1)
    assert beward._alarm_handlers == {logger1}

    beward.add_alarms_handler(logger1)
    assert beward._alarm_handlers == {logger1}

    beward.add_alarms_handler(logger2)
    assert beward._alarm_handlers == {logger1, logger2}


def test_remove_alarms_handler(beward) -> None:
    """Test that remove alarm handlers."""

    def logger1() -> None:
        pass

    def logger2() -> None:
        pass

    beward.add_alarms_handler(logger1)
    beward.add_alarms_handler(logger2)
    assert beward._alarm_handlers == {logger1, logger2}

    beward.remove_alarms_handler(logger1)
    assert beward._alarm_handlers == {logger2}

    beward.remove_alarms_handler(logger1)
    assert beward._alarm_handlers == {logger2}

    beward.remove_alarms_handler(logger2)
    assert beward._alarm_handlers == set()


def test__handle_alarm(beward) -> None:
    """Test that handle alarms."""
    # Check initial state
    assert beward.alarm_state == {ALARM_ONLINE: False}
    assert beward.alarm_timestamp == {ALARM_ONLINE: datetime.min}

    ts1 = datetime.now(local_tz)
    beward._handle_alarm(ts1, ALARM_MOTION, state=True)
    assert beward.alarm_state == {ALARM_ONLINE: False, ALARM_MOTION: True}
    assert beward.alarm_timestamp == {ALARM_ONLINE: datetime.min, ALARM_MOTION: ts1}

    ts2 = datetime.now(local_tz)
    beward._handle_alarm(ts2, ALARM_SENSOR, state=True)
    assert beward.alarm_state == {
        ALARM_ONLINE: False,
        ALARM_MOTION: True,
        ALARM_SENSOR: True,
    }
    assert beward.alarm_timestamp == {
        ALARM_ONLINE: datetime.min,
        ALARM_MOTION: ts1,
        ALARM_SENSOR: ts2,
    }

    ts3 = datetime.now(local_tz)
    beward._handle_alarm(ts3, ALARM_MOTION, state=False)
    assert beward.alarm_state == {
        ALARM_ONLINE: False,
        ALARM_MOTION: False,
        ALARM_SENSOR: True,
    }
    assert beward.alarm_timestamp == {
        ALARM_ONLINE: datetime.min,
        ALARM_MOTION: ts3,
        ALARM_SENSOR: ts2,
    }


def _listen_alarms_tester(alarms, expected_log) -> None:
    with requests_mock.Mocker() as mock:
        beward = BewardGeneric(MOCK_HOST, MOCK_USER, MOCK_PASS)
        mock.register_uri(
            "get", function_url("alarmchangestate"), text="\n".join(alarms)
        )
        log = []
        logging = True

        def _alarms_logger(device, timestamp, alarm, state) -> None:
            nonlocal logging
            if not logging:
                return

            if alarm == ALARM_ONLINE:
                log.append(";".join((str(alarm), str(state))))
                logging = state
            else:
                log.append(";".join((str(timestamp), str(alarm), str(state))))

        # Check initial state
        assert beward.alarm_state == {ALARM_ONLINE: False}
        assert beward.alarm_timestamp == {ALARM_ONLINE: datetime.min}

        alarms2listen = [x.split(";")[2] for x in alarms]
        beward.add_alarms_handler(_alarms_logger)
        beward.listen_alarms(alarms=alarms2listen)
        sleep(0.1)
        beward.remove_alarms_handler(_alarms_logger)
        sleep(0.1)

        expect = [
            "DeviceOnline;True",
        ]
        expect.extend(expected_log)
        expect.append("DeviceOnline;False")
        assert log == expect


def test_listen_alarms():
    """Test that listen alarms."""
    local_tz_str = datetime.now(local_tz).isoformat(timespec="seconds")[19:]

    alarms = ["2019-07-28;00:57:27;MotionDetection;1;0"]
    ex_log = [f"2019-07-28 00:57:27{local_tz_str};MotionDetection;True"]
    _listen_alarms_tester(alarms, ex_log)

    alarms.append("2019-07-28;00:57:28;MotionDetection;0;0")
    ex_log.append(f"2019-07-28 00:57:28{local_tz_str};MotionDetection;False")
    _listen_alarms_tester(alarms, ex_log)

    alarms.append("2019-07-28;15:51:52;SensorAlarm;1;0")
    ex_log.append(f"2019-07-28 15:51:52{local_tz_str};SensorAlarm;True")
    _listen_alarms_tester(alarms, ex_log)

    alarms.append("2019-07-28;15:51:53;SensorAlarm;0;0")
    ex_log.append(f"2019-07-28 15:51:53{local_tz_str};SensorAlarm;False")
    _listen_alarms_tester(alarms, ex_log)


def test_system_info():
    """Test that get system info from device."""
    data = load_fixture("systeminfo.txt")

    with requests_mock.Mocker() as mock:
        mock.register_uri("get", function_url("systeminfo"), text=data)
        beward = BewardGeneric(MOCK_HOST, MOCK_USER, MOCK_PASS)

        expect = {}
        for env in data.splitlines():
            (k, val) = env.split("=", 2)
            expect[k] = val

        assert beward.system_info == expect

        mock.register_uri(
            "get", function_url("systeminfo"), exc=requests.exceptions.ConnectTimeout
        )

        assert beward.system_info == expect  # Check for caching

        beward._sysinfo = None

        assert beward.system_info == {}


def test_device_type():
    """Test that detect device type."""
    with requests_mock.Mocker() as mock:
        mock.register_uri(
            "get", function_url("systeminfo"), text=load_fixture("systeminfo.txt")
        )
        beward = BewardGeneric(MOCK_HOST, MOCK_USER, MOCK_PASS)

        assert beward.device_type == BEWARD_DOORBELL

        mock.register_uri("get", function_url("systeminfo"), text="DeviceModel=DS03M")
        beward._sysinfo = None

        assert beward.device_type == BEWARD_DOORBELL

        mock.register_uri(
            "get", function_url("systeminfo"), text="DeviceModel=NONEXISTENT"
        )
        beward._sysinfo = None

        assert beward.device_type is None

        mock.register_uri("get", function_url("systeminfo"), text="NonExistent=DS03M")
        beward._sysinfo = None

        assert beward.device_type is None


def test_is_online():
    """Test that detect device is online."""
    data = load_fixture("systeminfo.txt")

    with requests_mock.Mocker() as mock:
        mock.register_uri("get", function_url("systeminfo"), text=data)

        beward = BewardGeneric(MOCK_HOST, MOCK_USER, MOCK_PASS)

        assert beward.is_online is True
        assert beward.available is True

        mock.register_uri("get", function_url("systeminfo"))

        assert beward.is_online is True
        assert beward.available is True

        mock.register_uri(
            "get", function_url("systeminfo"), exc=requests.exceptions.ConnectTimeout
        )

        assert beward.is_online is False
        assert beward.available is False



# pylint: disable=protected-access,redefined-outer-name
"""Test to verify that Beward library works."""

import contextlib
from typing import Any

from beward.util import is_valid_fqdn, normalize_fqdn


def test_normalize_fqdn():
    """Test that normalize FQDN."""
    fqdn = "google.com."
    expect = "google.com"
    assert normalize_fqdn(fqdn) == expect

    fqdn = "домен.рф."
    expect = "xn--d1acufc.xn--p1ai"
    assert normalize_fqdn(fqdn) == expect

    fqdn = "домен.рф.:80"
    expect = "xn--d1acufc.xn--p1ai"
    assert normalize_fqdn(fqdn) == expect


def test_is_valid_fqdn__label_too_long():
    """Test FQDN validations."""
    assertValidFQDN("b" * 63, "com")
    with contextlib.suppress(UnicodeError):
        assertInvalidFQDN("A" * 64, "com")
    with contextlib.suppress(UnicodeError):
        assertInvalidFQDN("b" * 63, "A" * 64, "com")


def test_is_valid_fqdn__name_too_long_254_octets():
    """Test FQDN validations."""
    parts = [(chr(ord("A") + i % 26)) for i in range(int(254 / 2) - 1)]
    parts.append("co")
    fqdn = ".".join(parts)
    assert len(fqdn) == 254
    assertInvalidFQDN(fqdn)


def test_is_valid_fqdn__name_ok_253_octets():
    """Test FQDN validations."""
    parts = [(chr(ord("A") + i % 26)) for i in range(int(254 / 2))]
    fqdn = ".".join(parts)
    assert len(fqdn) == 253
    assertValidFQDN(fqdn)


def test_rfc_1035_s_3_1__trailing_byte():
    """Test FQDN validations."""
    parts = [(chr(ord("A") + i % 26)) for i in range(int(254 / 2))]
    fqdn = ".".join(parts) + "."
    assert len(fqdn) == 254
    assertValidFQDN(fqdn)


def test_rfc_3696_s_2__label_invalid_starts_or_ends_with_hyphen():
    """Test FQDN validations."""
    assertInvalidFQDN("-a", "com")
    assertInvalidFQDN("a-", "com")
    assertInvalidFQDN("-a-", "com")
    assertInvalidFQDN("a", "-com")
    assertInvalidFQDN("a", "com-")


def test_rfc_3696_s_2__punycode():
    """Test FQDN validations."""
    assertValidFQDN("є", "com")
    assertValidFQDN("le-tour-est-joué", "com")
    assertValidFQDN("invalid", "cóm")
    assertValidFQDN("ich-hätte-gern-ein-Umlaut", "de")


def test_rfc_3696_s_2__preferred_form_invalid_chars():
    """Test FQDN validations."""
    assertInvalidFQDN("\x01", "com")
    assertInvalidFQDN("x", "\x01\x02\x01")


def test_rfc_3696_s_2__valid():
    """Test FQDN validations."""
    assertValidFQDN("shopping", "on", "the", "net")
    assertValidFQDN("who", "is")
    assertValidFQDN("bbc", "co", "uk")
    assertValidFQDN("example", "io")


def test_rfc_3696_s_2__invalid():
    """Test FQDN validations."""
    assert is_valid_fqdn("192.168.0.1") is False


def _is_valid_fqdn_from_labels_sequence(fqdn_labels_sequence) -> bool:
    fqdn = ".".join(fqdn_labels_sequence)
    return is_valid_fqdn(fqdn)


# pylint: disable=invalid-name
def assertValidFQDN(*seq: Any) -> None:  # noqa: N802
    """Positive assert function for FQDN validations."""
    assert _is_valid_fqdn_from_labels_sequence(seq) is True


# pylint: disable=invalid-name
def assertInvalidFQDN(*seq: Any) -> None:  # noqa: N802
    """Negative assert function for FQDN validations."""
    assert _is_valid_fqdn_from_labels_sequence(seq) is False



#  Copyright (c) 2019-2022, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
"""Beward doorbell controller."""

import logging
from datetime import datetime
from typing import Any

from beward.const import ALARM_SENSOR

from .camera import BewardCamera

_LOGGER = logging.getLogger(__name__)


class BewardDoorbell(BewardCamera):
    """Beward doorbell controller class."""

    def __init__(self, host: str, username: str, password: str, **kwargs: Any) -> None:
        """Initialize Beward doorbell controller."""
        super().__init__(host, username, password, **kwargs)

        self.last_ding_timestamp = None
        self.last_ding_image = None

    def _handle_alarm(self, timestamp: datetime, alarm: str, state: bool) -> None:  # noqa: FBT001
        """Handle alarms from Beward device."""
        super()._handle_alarm(timestamp, alarm, state)

        if alarm == ALARM_SENSOR and state:
            self.last_ding_timestamp = timestamp
            self.last_ding_image = self.live_image



#!/usr/bin/env python
"""Library module setup."""

import re
from pathlib import Path

from setuptools import find_packages, setup


def load_requirements(fpath: str) -> list:
    """Load requirements from file."""
    with Path(fpath).open(encoding="utf8") as f_req:
        data = list(f_req)
    imp = re.compile(r"^(-r|--requirement)\s+(\S+)")
    reqs = []
    for i in data:
        # pylint: disable=invalid-name
        m = imp.match(i)
        if m:
            reqs.extend(load_requirements(m.group(2)))
        else:
            reqs.append(i)

    return reqs


PACKAGES = [x for x in find_packages() if x not in ["scripts", "tests"]]

with (Path(PACKAGES[0]) / "const.py").open(encoding="utf-8") as file:
    src = file.read()
metadata = dict(re.findall(r'([a-z]+) = "([^"]+)"', src, re.IGNORECASE))
metadata.update(dict(re.findall(r"([a-z]+) = '([^']+)'", src, re.IGNORECASE)))
docstrings = re.findall(r'"""(.*?)"""', src, re.MULTILINE | re.DOTALL)

VERSION = metadata["VERSION"]

REQUIREMENTS = load_requirements("requirements.txt")

setup(
    version=VERSION,
    packages=PACKAGES,
    install_requires=REQUIREMENTS,
)



# pylint: disable=protected-access,redefined-outer-name
"""Test to verify that Beward library works."""

from datetime import datetime

import requests_mock
from requests import ConnectTimeout

from beward import BewardCamera
from beward.const import ALARM_MOTION

from . import function_url, load_binary, load_fixture
from .const import MOCK_HOST, MOCK_PASS, MOCK_USER, local_tz


def test___init__() -> None:
    """Initialize test."""
    beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)
    assert beward.rtsp_port is None
    assert beward.stream == 0

    beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS, rtsp_port=123, stream=2)
    assert beward.rtsp_port == 123
    assert beward.stream == 2


def test_obtain_uris() -> None:
    """Test that obtain urls from device."""
    with requests_mock.Mocker() as mock:
        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)

        assert beward._live_image_url is None
        assert beward._rtsp_live_video_url is None

        mock.register_uri("get", function_url("rtsp"), exc=ConnectTimeout)
        beward.obtain_uris()

        expect = (
            "rtsp://" + MOCK_USER + ":" + MOCK_PASS + "@" + MOCK_HOST + ":554/av0_0"
        )
        assert beward._rtsp_live_video_url == expect

        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS, stream=1)
        mock.register_uri("get", function_url("rtsp"))
        beward.obtain_uris()

        expect = (
            function_url("images", user=MOCK_USER, password=MOCK_PASS) + "?channel=0"
        )
        assert beward._live_image_url == expect

        expect = (
            "rtsp://" + MOCK_USER + ":" + MOCK_PASS + "@" + MOCK_HOST + ":554/av0_1"
        )
        assert beward._rtsp_live_video_url == expect

        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)
        mock.register_uri("get", function_url("rtsp"), text=load_fixture("rtsp.txt"))
        beward.obtain_uris()

        expect = (
            "rtsp://" + MOCK_USER + ":" + MOCK_PASS + "@" + MOCK_HOST + ":47456/av0_0"
        )
        assert beward._rtsp_live_video_url == expect

        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS, rtsp_port=123, stream=2)
        mock.register_uri("get", function_url("rtsp"), text=load_fixture("rtsp.txt"))
        beward.obtain_uris()

        expect = (
            "rtsp://" + MOCK_USER + ":" + MOCK_PASS + "@" + MOCK_HOST + ":123/av0_2"
        )
        assert beward._rtsp_live_video_url == expect


def test_live_image_url() -> None:
    """Test that obtain live image url from device."""
    with requests_mock.Mocker() as mock:
        mock.register_uri("get", function_url("rtsp"))
        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)

        assert beward._live_image_url is None

        expect = (
            function_url("images", user=MOCK_USER, password=MOCK_PASS) + "?channel=0"
        )
        assert beward.live_image_url == expect


def test_rtsp_live_video_url() -> None:
    """Test that obtain RTSP live video url from device."""
    with requests_mock.Mocker() as mock:
        mock.register_uri("get", function_url("rtsp"), text=load_fixture("rtsp.txt"))
        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)

        assert beward._rtsp_live_video_url is None

        expect = (
            "rtsp://" + MOCK_USER + ":" + MOCK_PASS + "@" + MOCK_HOST + ":47456/av0_0"
        )
        assert beward.rtsp_live_video_url == expect


def test_live_image() -> None:
    """Test that receive live image from device."""
    image = load_binary("image.jpg")

    with requests_mock.Mocker() as mock:
        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)

        mock.register_uri("get", function_url("images"), content=image)
        res = beward.live_image
        assert res is None

        mock.register_uri(
            "get",
            function_url("images"),
            content=image,
            headers={"Content-Type": "image/jpeg"},
        )
        res = beward.live_image
        assert res == image


def test__handle_alarm() -> None:
    """Test that handle alarms."""
    image = load_binary("image.jpg")

    with requests_mock.Mocker() as mock:
        beward = BewardCamera(MOCK_HOST, MOCK_USER, MOCK_PASS)

        # Check initial state
        assert beward.last_motion_timestamp is None
        assert beward.last_motion_image is None

        ts1 = datetime.now(local_tz)
        mock.register_uri(
            "get",
            function_url("images"),
            content=image,
            headers={"Content-Type": "image/jpeg"},
        )
        beward._handle_alarm(ts1, ALARM_MOTION, state=True)
        assert beward.last_motion_timestamp == ts1
        assert beward.last_motion_image == image
